import pandas as pd 
import numpy as np 
from .preprocessing import *
from .Train import *


class Damma():
    '''
    Downscalling de campos oriundos do wrf 
    '''
    def fit(self,VAR, d1 , d2 ,init , end, regressor):
        '''
        Realiza o downscaling estatistico dos dados de saida do wrf 
        
        input:
            VAR: Váriavel de interesse no arquivo netCDF do WRF
            d1: output do wrf referente ao dom inicial
            d2: output do wrf referente ao dom final na escala espacial que se deseja realizar o downscaling
            init: passo inicial tendo por base os arquivos netcdf
            end: passo final
            regressor: estimador a ser utilizado, estruturado de forma "model.fit"
            
        '''
        # abrindo  nc
        dt1 = dataset(d1)
        dt2 = dataset(d2)
        
        # separando variaveis Di
        
        mde1 = dt1.mde()
        self.mdeD1 = mde1.flatten()
        self.lulcD1 = dt1.landUse().flatten()
        self.latD1 = dt1.xlat().flatten()
        self.lonD1 = dt1.xlong().flatten()
        slopeD1 , aspectD1 = slope_aspect(mde1)
        self.slopeD1 = slopeD1.flatten()
        self.aspectD1 = aspectD1.flatten()
        
        # criando DataFrame Di
        lista = [self.mdeD1,self.lulcD1,self.latD1,self.lonD1,self.slopeD1,self.aspectD1]
        nomes = [ 'topo', 'lu_index','xlat','xlong', 'slope', 'aspect']
        domiX = pandasDF(lista,nomes)
        
        t2md1 = dt1.getVar(VAR)
        self.varD1 = t2md1[init : end]
        
        # separando variaveis Df
        mde2 = dt2.mde()
        self.mdeD2 = mde2.flatten()
        self.lulcD2 = dt2.landUse().flatten()
        self.latD2 = dt2.xlat().flatten()
        self.lonD2 = dt2.xlong().flatten()        
        slopeD2 , aspectD2 = slope_aspect(mde2)
        self.slopeD2 = slopeD2.flatten()
        self.aspectD2 = aspectD2.flatten()
        
        # criando DataFrame Df
        lista = [self.mdeD2,self.lulcD2,self.latD2,self.lonD2,self.slopeD2,self.aspectD2]
        nomes = [ 'topo', 'lu_index','xlat','xlong', 'slope', 'aspect']
        domfX = pandasDF(lista,nomes)
        
        t2md2 = dt2.getVar(VAR)
        self.varD2 = t2md2[init : end]
        
        # resolvendo conflitos de classes de uso do solo
        domiX.lu_index[domiX['lu_index']==6]=7
        domiX.lu_index[domiX['lu_index']==16]=10
        
        domfX.lu_index[domfX['lu_index']==6]=7
        domfX.lu_index[domfX['lu_index']==16]=10
        
        

        # aplicando downscaling
        # Padroniza
        domiX = X_structure(domiX)
        domfX = X_structure(domfX)
        self.dom_i = PreProcessingX(domiX)
        self.dom_f = PreProcessingX(domfX)
        
        
        self.Ypred , self.regressor = train(self.varD1, self.dom_i, self.dom_f, model = 3, regressor=regressor)
        
        
    

        
        
        



import pandas as pd 
import numpy as np

'''
Manipulacao e testes no dado de saida 
'''

#from netCDF4 import Dataset
import pandas as pd


def getXYcoordinate(lat,lon,nc = None, XLAT=None, XLONG=None ):
    '''
    Retorna a coordenadas x,y de uma respectiva grade a partir da lat e long
    input:
        lat = latitude do ponto
        lon = longitude do ponto
        nc = arquivo wrfoutput em formato netCDF
        XLAT = latitude 2D
        XLONG = longitude 2D
    
    output:
        x , y  coordenadas
        
    exemplo:
        x, y = getXYcoordinate(-43.43,-22.32, nc = nc)
        x, y = getXYcoordinate(-43.43,-22.32, XLAT = XLAT, XLONG = XLONG)        
    '''
    try:
        if XLAT==None and XLONG==None:
            cor_lat = pd.DataFrame(nc.variables['XLAT'][0][:])
            cor_lat2 = pd.DataFrame({'a': cor_lat.iloc[:, 0], 'b': abs(cor_lat.iloc[:, 0] - lat)})
            x = cor_lat2[cor_lat2.b == min(cor_lat2.b)].index.to_numpy()[0]

            cor_lon = pd.DataFrame(nc.variables['XLONG'][0][:])
            cor_lon2 = pd.DataFrame({'a': cor_lon.iloc[0, :], 'b': abs(cor_lon.iloc[0, :] - lon)})
            y = cor_lon2[cor_lon2.b == min(cor_lon2.b)].index.to_numpy()[0]
            
        else:
            cor_lat = pd.DataFrame(XLAT)
            cor_lat2 = pd.DataFrame({'a': cor_lat.iloc[:, 0], 'b': abs(cor_lat.iloc[:, 0] - lat)})
            x = cor_lat2[cor_lat2.b == min(cor_lat2.b)].index.to_numpy()[0]

            cor_lon = pd.DataFrame(XLONG)
            cor_lon2 = pd.DataFrame({'a': cor_lon.iloc[0, :], 'b': abs(cor_lon.iloc[0, :] - lon)})
            y = cor_lon2[cor_lon2.b == min(cor_lon2.b)].index.to_numpy()[0]
            
    except:
        print( 'Insira um arquivo netcdf wrfouput ou os campos xlat e xlong em 2d numpy array')

    return  x , y

def extractpoint(x, y, array, convert = True):
    '''
    A partir das coordenadas de grade x y extrai serie historica do ponto de grade para uma variavel desejada numa numpy 3d array
    input:
        x ,y = coordenadas do ponto de grade
        array = numpy array 3d com indices t,x,y, onde t e o tempo        
        convert = Se True, converte o campo de temperatura de kelvin para graus celsios
    '''

    if convert==True:
        vlr = array[:, x, y]-273.15 #This change from kelvin to celsius

    else:
        vlr = array[:, x, y] #This change from kelvin to celsius

    return vlr


def extractpointNCDF(x, y, nc,  variavel = 'T2', convert = True):
    '''
    A partir das coordenadas de grade x y extrai serie historica do ponto de grade para uma variavel desejada num arquivo netcdf
    input:
        x ,y = coordenadas do ponto de grade
        nc = Arquivo netcdf desejado
        variavel = varaivel do netCDF desejada
        convert = Se True, converte o campo de temperatura de kelvin para graus celsios
    '''

    if convert==True:
        vlr = (nc.variables[variavel][:, x, y]-273.15)[:] #This change from kelvin to celsius

    else:
        vlr = (nc.variables[variavel][:, x, y])[:] #This change from kelvin to celsius

    return vlr


def extractvariavel(netCDFfile,variavel = 'T2'):
    nc = Dataset(netCDFfile)
    variavel = (nc.variables[variavel][:])[:]
    return variavel


def DataframePoints(est,simulacao, nc,start = '2018-01-17 00:00:00'):
    '''
    Retorna um dataframe com a variavel do campo simulacao para cada estacao meteorologica
    input:
        est: dataframe com os atributos Estacoes (nome da estacao),LAT e LON correspondente
        simulacao: array 3d da variavel de interesse
        nc: wrfouput em formato netcdf
        start: data inicial das simulacoes
        
    output:
        pandas dataframe: Columns = nome das estacoes, Linhas = data/hora correspondente
    '''
    estacoes = est.Estacoes
    lat = est.LAT.values
    lon = est.LON.values
    VAR = {}    

    for i , e in enumerate(estacoes):
        x , y = getXYcoordinate(lat[i],lon[i],nc)
        vlr = extractpoint(x,y,simulacao)        
        VAR[estacoes[i]] = vlr
    
    index = pd.date_range(start = start, freq = 'H',periods = len(simulacao))
    VAR = pd.DataFrame.from_dict(VAR)
    VAR.index = index
    
    return VAR
    
    



import pandas as pd
import numpy as np
from .estatisticas import *
import time
from datetime import datetime
from .preprocessing import *

'''
Algoritmos de treino e de teste 
'''

def test_carregamento():
    print('teste 4')


class ModelCreate():
    def __init__(self, model, pre_process = None, filter_water=True, wrfoutput = None ):
        self.model = model
        self.pre_process_base = pre_process
        self.filter_water = filter_water
        self.train_metrics = pd.DataFrame()
        
        if wrfoutput is not None:
            self.database = DataBase(wrfoutput)
            self.mask_lulc = np.array(self.database.lulc==17)
            print('Base carregada')
        
    # create grade:
    @staticmethod
    def GradeCreatorDom_f(Y,shape = (85,195)):
        '''
        cria a grade no domínio do shape l,c
          
        Y: array com dados
        shape = dimensoes (l,c) l - linhas, c - colunas
        '''
        l = shape[0]
        c = shape[1]
        t2 = Y.reshape(l,c)
        return t2
    
    def _dataframe_transform(self,df):
        aux = pd.DataFrame()
        for i in df.keys():
            aux[i] = df[i].flatten()
        return aux
    
    def predict(self,x_test):
        """
        Return y_pred values to model
        -------------------------------------
        Input: pd.Dataframe
        
            x_test: values x to predict
        output:
            pd.DataFrame whith MSE, RMSE, r, r2 and NS
        """  
                #x_test = self.preprocesss_step(x_test)

        
        # aplicando o modelo
        shp = x_test[list(x_test.keys())[0]].shape
        
        x_test = self._dataframe_transform(x_test)
        
        
        #retirando pontos com agua
#         if self.filter_water:
#             #x_test['id']  = x_test.index
#             idW = x_test.loc[self.mask_lulc].index
#             idNW = x_test.loc[~self.mask_lulc].index
#             x_test = x_test.loc[~self.mask_lulc]
        

        y_pred = self.model.predict(x_test)
        # criando saida 
#         if self.filter_water:
#             df_aux = self._mask_water_apply(idNW,idW,'VAR',y_pred)
#             y_pred = df_aux['VAR'].values
        try:
            y_pred = y_pred.reshape(shp[0],shp[1])
            print('campo reorganizado')
        except:
            print('Vetor unico')
        return y_pred
        
    def _mask_water_apply(self, idNW,idW,var_name,var_data):
        df_agua = pd.DataFrame({'id':idW, var_name: np.full([idW.shape[0]], np.nan)})
        df_sem_agua = pd.DataFrame({'id':idNW, var_name: var_data})
        df_aux = df_agua.append(df_sem_agua)
        df_aux = df_aux.sort_values(['id'])
        
        return df_aux[var_name]
        
    def metricas(self,s,o):        
        """
        Return MSE, RMSE, r, r2 and NS
        nput:
            s: simulated
            o: observed
        output:
            MSE, RMSE, r, r2 and NS
        """  
        Mae = mae(s,o)
        Mse = mse(s,o)
        Rmse = rmse(s,o)
        R_coef = r(s,o)
        R_2   = r2(s,o)
        NSE = NS(s,o)

        return [Mae,Mse, Rmse, R_coef, R_2, NSE]

    def preprocesss_step(self,df):
        if self.pre_process_base is not None:
            df = self.pre_process_base.fit_transform(df)
            return df
        else:
            return df
        
    
    def fit(self, x, y,index_name='Ensaio'):
        ti = time.time()
        
#       # Dataframe tranform
        x_train = self._dataframe_transform(x)
        
        y=y.flatten()

        
        if self.filter_water:
            x_train['Y'] = y
            y = x_train['Y'][~self.mask_lulc]
            x_train = x_train[~self.mask_lulc]
            x_train.drop('Y',axis=1,inplace=True)
        
        # Treinando o modelo
        self.model.fit(x_train,y)
        y_train = self.model.predict(x_train)
        self.x_train = x_train
        self.train_metrics = pd.concat([self.train_metrics,errcoef(y,y_train,index_name)])
        
        
         
    
    
    def fit_batch(self,X_base,x_test,delta, target):
        inicio = datetime.now()
        
        self.target_base = self.database.get_var_din(target,steps=delta)        
        
        lista = []
        for i in range(self.target_base.shape[0]):
            print(i) 
            print(self.target_base.shape)
            
            y_train = self.target_base[i].flatten()
            print(f'shape y: {y_train.shape[0]}')
            #assert X_base[list(X_base.keys())[0]].flatten().shape[0]==y_train.shape[0]
            
            self.fit(X_base,y_train,index_name='time_pass_'+str(i))
            y_pred = self.predict(x_test)
            lista.append(y_pred)
            
        self.test_result = np.array(lista)
        fim = datetime.now()
        print(f'tempo de processamento: {fim - inicio}')
    
    @staticmethod       
    def get_icu(lclu, T, shape=(85,195)):
        """
        Return Urban Heat island from T domain 
        
        -------------------------------------------
        inputs:
            lclu: pd.Series, Array(1D or 2D), list
            land canopy and land use where int 13 is urban use.
            
        outputs:array 2
            maps with Urban Heat Island
            
        """
        if isinstance(lclu, pd.DataFrame):
            lclu = lclu.values
        else:
            lclu= np.array(lclu)
        
        T = T.flatten()
        t_rural_mean = T[(lclu!=13) & (lclu!=17)].mean()
        T_icu = T - t_rural_mean
        
        return ModelCreate.GradeCreatorDom_f(T_icu, shape=shape)
            
            
            


import pandas as pd
import numpy as np

'''
Fornece ferramentas estatisticas para avaliar o desempenho dos modelos, tendo como base dados observados e modelados
'''
# funcoes
def remove_nan(o,s):
    '''
    y o valor '''
    o = np.array(o)
    s = np.array(s)
    o = o[~np.isnan(s)]
    s = s[~np.isnan(s)]
    return s, o

def rmse(s,o):    
    """
    Raiz quadrada do erro medio 
    input:
        s: simulated
        o: observed
    output:
        rmses: root mean squared error
    """
    s,o = remove_nan(o,s)
    
    return np.sqrt(np.mean((s-o)**2))

def mae(s,o):
    """
    Erro medio absoluto
    input:
        s: simulated
        o: observed
    output:
        maes: mean absolute error
    """
    s,o = remove_nan(o,s)
    return np.mean(abs(s-o))

def mse(s,o):
    """
    Erro quadratico medio 
    input:
        s: simulated
        o: observed
    output:
        maes: mean squared error
    """
    s,o = remove_nan(o,s)
    return np.mean((s-o)**2)

def r2(s,o):
    """ 
    Coeficiente de determinacao
    input: 
        s: simulated
        o: observed
    output:
        r2: Coefficient of determination
        """
    s,o = remove_nan(o,s)
    return (np.corrcoef(s,o)[0,1])**2

def r(s,o):
    """ 
    coeficiente de correlacao
    input: 
        s: simulated
        o: observed
    output:
        r: Coefficient of correlation
    """
    s,o = remove_nan(o,s)
    return np.corrcoef(o,s)[0,1]

def mape(s,o):
    s,o = remove_nan(o,s)
    mape_v = abs((abs(o-s).sum()/o.sum()))
    
    return mape_v


def accuracy(mape_v):
    return 1 - mape_v
    

def NS(s,o):
    """
    coeficiente de eficiência do modelo de Nash
    input:
        s: simulated
        o: observed
    output:
        ns: Nash Sutcliffe efficient coefficient
    """   
    s,o = remove_nan(o,s)
    return 1 - sum((s-o)**2)/sum((o-np.mean(o))**2)

# main function
def errcoef(s,o,nome='Teste'):
    """
    Return MSE, RMSE, r, r2 and NS
    nput:
        s: simulated
        o: observed
    output:
        MSE, RMSE, r, r2 and NS
    """     
    MAPE = mape(s,o) 
    Facc = accuracy(MAPE)
    Mae = mae(s,o)
    Mse = mse(s,o)
    Rmse = rmse(s,o)
    R_coef = r(s,o)
    R_2   = r2(s,o)
    NSE = NS(s,o)
    #Mae,Mse, Rmse, R_coef, R_2, NSE
    df = pd.DataFrame({'MAE':Mae,'MSE':Mse,
                       'RMSE':Rmse,'R':R_coef,
                       'R2':R_2, 'NSE':NSE,
                       'MAPE':MAPE,'Facc':Facc}, index = [nome])
    
    return df


    
def avaliando(s,o):
    """ 
    retorna um pandas dataframe com coeficientes de erro
    input:
        s: dataframe com valores simulados
        o: dataframe com valores observados
    output: 
        df: dataframe com os erros 
    """
    estacoes = s.columns
    df = pd.DataFrame()
        

    for i in estacoes:
        w = errcoef(s[i].values, o[i].values,i)
        df = df.append(w)
    
    return df
        
def coefsplit(lista, nomes):
    nse = pd.DataFrame()
    mse = pd.DataFrame()
    rmse = pd.DataFrame()
    r = pd.DataFrame()
    r2 = pd.DataFrame()
    for k , i in enumerate(lista):
        nse[nomes[k]] = i['NSE']
        mse[nomes[k]] = i['MSE']
        rmse[nomes[k]] = i['RMSE']
        r[nomes[k]] = i['R']
        r2[nomes[k]] = i['R2']
       
    
    return nse, mse, rmse, r, r2


from netCDF4 import Dataset 
import numpy as np
import pandas as pd


#------------------------------------------------------------------------
"""
Isola variaveis dependentes e independentes do arquivo wrfoutput

"""
import matplotlib.tri as tri
import matplotlib.pyplot as plt
#import geopandas as geo
#lulc= pd.read_json('../Dados/interin/X_d3.json')
#rj = geo.read_file('../../../dados_informacoes/vetores/RMRJ/RMRJ.shp')


def plot_map(campo,figsize=(10,5),
             cmap='jet',title = None, 
             lat=None,lon=None, 
             linecolor='lightgrey',
             vetor = None,
             ax=None,
             mask_water=False,
             lulc=None
            ):

    campo = campo.flatten()
    modelo = np.nan_to_num(campo,nan=0)
    
    triang = tri.Triangulation(lon,lat)
    if mask_water:
        mask = (lulc==17).flatten()
        mask = np.all(np.where(mask[triang.triangles], True, False), axis=1)
        triang.set_mask(mask)
    
    if ax is None:
        fig, ax = plt.subplots(1,1,figsize=figsize)

    
    cax = ax.tricontourf(triang,modelo,cmap=cmap)
    ax.set_xlabel('Long',fontsize=14)
    ax.set_ylabel('Lat',fontsize=14)
    #ax.set_ylim(-23.2,-22.5)
    plt.xticks(fontsize=12)
    plt.yticks(fontsize=12)
    
    #fig.colorbar(cax,label='Intensidade das ICUs',ax=ax)
    plt.colorbar(cax, ax=ax, shrink=0.7)
    
    #rj.plot(facecolor='none', edgecolor= linecolor, lw =0.7,ax=ax)
    
    if title is not None:
        ax.set_title(title)  



def rotT2(T2):
    """Rotaciona a imagem T2 do wrf para um o padrao Norte superior"""
    Rep = np.rot90(T2,2)
    Rep = np.flip(Rep,1)
    return Rep

class DataBase:
    def __init__(self,wrfoutput):
        """
        Abre o arquivo netcdf wrfout(..) e extrai as variaveis de interese
        input: 
            wrfouput: nome ou diretorio para o arquvio wrfout(..) entre aspas
        """
        self.wrfoutput = Dataset(wrfoutput)
        try:
            self.lulc = self.get_var_static('LU_INDEX').flatten()
            self.lat = self.get_var_static('XLAT').flatten()
            self.lon = self.get_var_static('XLONG').flatten()
            
        except:
            print('Lat and Lon info not loading!')
        
    @property
    def list_variables(self):
        return self.wrfoutput.variables.keys()
      
    def Temperatura(self):
        """
        Retorna a temperatura a 2m referente ao aquivo wrfouput 
        """        
        self.T2m = self.wrfoutput.variables['T2'][:]
        t2rot = []
        for i , n in enumerate(self.T2m):
            campo = rotT2(n)
            t2rot.append(campo)
        self.T2m = np.array(t2rot)
        return self.T2m
    
    def get_var_din(self, name, steps=None):
        """
        get dynamic variable in database 
        
        name: str , default obligatory
            variable name
        
        steps:  tuple , default None
            temporal range of the variable select
        """        
        if steps is None:
            self.var = self.wrfoutput.variables[name][:]
        else:
            self.var = self.wrfoutput.variables[name][steps[0]:steps[1]]
        
        varRot = []
        for i , n in enumerate(self.var):
            campo = rotT2(n)
            varRot.append(campo)
        varGet = np.array(varRot)
        return varGet
    
    def get_var_static(self,name):
        """
        get static variable 
        """
        self.static = self.wrfoutput.variables[name][1]
        return rotT2(self.static)
    

    def get_table_independent_var(self, variables:list = None):
        df = {}#pd.DataFrame()
        for i in variables:
            var = np.array(self.get_var_static(i))#.flatten()
            df[i] = var
        return df
    
    def get_times_periods(self):
        d0 = pd.to_datetime(self.wrfoutput.SIMULATION_START_DATE,format='%Y-%m-%d_%H:%M:%S')
        serie = pd.date_range(start=d0,freq='H',periods=self.wrfoutput.variables['Times'].shape[0])
        return serie
        

    def plot(self,campo, water_mask=True, **kw):
        plot_map(campo,lat=self.lat,lon=self.lon, mask_water=water_mask, lulc=self.lulc, **kw)
    

    
####  variaveis do terreno
def slope_aspect(mde):    
    '''
    Retorno a declividade e aspecto do terreno
    input:
        mde: array do modelo digital de elevacao 
        
    output:
        slope: 2D array 
        aspect: 2D array  
    '''
    x, y = np.gradient(mde)
    slope = np.pi/2. - np.arctan(np.sqrt(x*x + y*y))
    aspect = np.arctan2(-x, y)
    return np.array(slope) , aspect

def hillshade(array, azimuth, angle_altitude):

    x, y = np.gradient(array)
    slope = np.pi/2. - np.arctan(np.sqrt(x*x + y*y))
    aspect = np.arctan2(-x, y)
    azimuthrad = azimuth*np.pi / 180.
    altituderad = angle_altitude*np.pi / 180.


    shaded = np.sin(altituderad) * np.sin(slope) \
     + np.cos(altituderad) * np.cos(slope) \
     * np.cos(azimuthrad - aspect)
    return 255*(shaded + 1)/2


# ------------------ Criando dataframe Test e treino -------------------
#funcoes
def flatten(array):
    '''
    traforma uma array nD numa array 1D 
    input: 
        array: numpy array nDimensional
    output:
        array numpy 1D'''
    
    return array.flatten()
    

def time_index(start='2017-12-17  00:00:00', periods=67):    
    """ 
    Define o Dataindex para o passo X, considerando o intervalo do estudo
    necessario ajustar o inicio para i periodo desejado
    input:
        start: data e hora de inicio
        periods = numero de periodos desejados 
        
    output:
        time serie
    """
    time = pd.date_range(start = start,
                  periods=periods,freq='H')
    return time

def pandasDF(lista, nomes,start='2017-12-17  00:00:00'):
    '''
    Cria um pandas daframe a partir de uma lista com as variaveis de interesse
    input:
        lista: lista/tupla com variaveis de interesse em 2d
        nomes: lista com nomes das variaveis de interesse
    
    output:
        Pandas Daframe com variaveis de interesse     
    '''
    df = pd.DataFrame()
    for i , var in enumerate(lista):
        df[nomes[i]] = var
        
    
    return df
        
        


from setuptools import setup

with open("README.md", "r") as fh:
    readme = fh.read()

setup(name='WRFdownscalingML',
    version='0.0.2',
    url='https://https://github.com/rjsampa/WRFdownscalingML',
    license='MIT License',
    author='Rafael Sampaio',
    long_description=readme,
    long_description_content_type="text/markdown",
    author_email='samprafael@gmail.com',
    keywords='Pacote',
    description=u'Framework para downscaling de outputs do WRF',
    packages=['WRFdownscalingML'],
    install_requires=['numpy','pandas','matplotlib','netCDF4'],)




import xgboost as xgb
from bayes_opt import BayesianOptimization
import pandas as pd

from datetime import datetime
        
class optimizeXGBClassifierBO():   
    def __init__(self,x_train, x_test,y_train,y_test, verbose=0):
        '''
    Classe para optmizacao do XGB utilizando optimizacao Bayesiana[1]. Tem por base o XGBClassifier. 
    
    inputs:
        x_train: pd.Dataframe - Dataframe com as features para treino
        x_test: pd.Dataframe - Dataframe com as features para teste
        y_train: pd.Dataframe - Dataframe com o target para treino
        y_test: pd.Dataframe - Dataframe com o target para teste
    
    methods:
        fit: executa a optmizacao
        XGB_BO: class - BayesianOptimization class
        params: dict - parametros do melhor modelo
        estatistica_treino: tuple - metricas (KS, Gini, AUC) do melhor modelo para treino 
        estatistica_teste: tuple -  metricas (KS, Gini, AUC) do melhor modelo para teste 
        
    
    -------------------------------------------------------------------------------------------------
    Exemplo:
    
        X_train, X_test, y_train, y_test = train_test_split(x,y,test_size = 0.30, random_state=42)
        optimizador = optimizeXGBClassifierBO(X_train,X_test,y_train,y_test)
        optimizador.fit(n_iter=30)
        resultados = pd.DataFrame(optimizador.XGB_BO.res).sort_values('target',ascending=False)

    
    
        
    1 - https://github.com/fmfn/BayesianOptimization       
    ''' 
        self.dtrain = xgb.DMatrix(x_train, label=y_train)
        del(x_train)
        self.dtest = xgb.DMatrix(x_test)
        del(x_test)
        self.y_train = y_train
        self.y_test = y_test
        self.verbose = verbose
        
        
    def xgb_evaluate(self,max_depth,gamma,colsample_bytree,min_child_weight,max_delta_step,subsample,eta,learning_rate,scale_pos_weight,reg_alpha,reg_lambda):
        T1 = datetime.now()
        params = {'eval_metric': 'rmse',
                   'objective' : 'reg:squarederror',
                  'nthread' : 4,
                  'max_depth': int(max_depth),
                  #'subsample' : max(min(subsample, 1), 0),
                  'subsample': subsample,
                  'eta': eta,
                  'gamma': gamma,
                  'min_child_weight' : min_child_weight,
                  'max_delta_step' : int(max_delta_step),
                  'colsample_bytree': colsample_bytree,
                  'learning_rate':learning_rate,
                  'scale_pos_weight':scale_pos_weight,
                   'reg_alpha':reg_alpha,
                   'reg_lambda':reg_lambda
                   
                  }
        # Used around 1000 boosting rounds in the full model
        cv_result = xgb.cv(params, 
                           self.dtrain, 
                           num_boost_round=100, 
                           metrics='rmse', 
                           nfold=3)

        T2 = datetime.now()

        # Bayesian optimization only knows how to maximize, not minimize, so return the negative RMSE
        return -1.0*cv_result['test-rmse-mean'].iloc[-1]
    
    def fit(self,n_iter=5, params={'max_depth': (5, 9),
                                   'gamma': (0.001, 1),
                                   'min_child_weight': (5, 20),
                                   'scale_pos_weight': (1.2, 5),
                                   'reg_alpha': (4.0, 7.0),
                                   'reg_lambda': (1.0, 7.0),
                                   'max_delta_step': (1, 5),
                                   'subsample': (0.4, 1.0),
                                   'colsample_bytree': (0.3, 1.0),
                                   'eta': (0.1, 1.0),
                                   'learning_rate': (0.1, 1.0)}, return_metrics=False):
     
        
        """
        Realiza optimizacao
        input:
            n_iter: int - Numero de interacoes 
        """
        
        
#         {'max_depth': (2, 7), # Máximo 7 para implantação. Não ultrapassar 7!
#                                    'gamma': (0.001, 10.0),
#                                    'min_child_weight': (0, 20),
#                                    'max_delta_step': (0, 10),
#                                    'subsample': (0.4, 1.0),
#                                    'colsample_bytree':(0.4, 1.0),
#                                   }
        
        
        self.XGB_BO = BayesianOptimization(self.xgb_evaluate, 
                                           params, verbose=self.verbose
                                          )
        
        self.XGB_BO.maximize(init_points=3,n_iter=n_iter,acq='ei')
        
        self.params = pd.DataFrame(self.XGB_BO.res).sort_values('target', ascending = False).iloc[0,1]
        self.params['max_depth'] = int(self.params['max_depth'])
        
#         print('-'*130)
#         print('Final Results')
#         print('Best XGBOOST parameters: ', self.params)
        
        
        # Train a new model with the best parameters from the search
        #model2 = xgb.train(self.params, self.dtrain, num_boost_round=250)

        # Predict on testing and training set
        #y_pred_test = model2.predict(self.dtest)
        #y_pred_train = model2.predict(self.dtrain)

        
        

        
#--------------------------------------------------------------------------------------------------------------
# Baseado em https://www.kaggle.com/sz8416/simple-bayesian-optimization-for-lightgbm

import lightgbm as lgb
from bayes_opt import BayesianOptimization
import numpy as np

def bayes_parameter_opt_lgb(X, y, init_round=15, opt_round=25, n_folds=5, random_seed=0, n_estimators=10000, learning_rate=0.05):
    # prepare data
    train_data = lgb.Dataset(data=X, label=y, free_raw_data=False)
    # parameters

    def lgb_eval(num_leaves, feature_fraction, max_depth , min_split_gain, min_child_weight):
        params = {
            "objective" : "regression", "bagging_fraction" : 0.8, "bagging_freq": 1,
            "min_child_samples": 20, "reg_alpha": 1, "reg_lambda": 1,"boosting": "rf",
            "learning_rate" : 0.01, "subsample" : 0.8, "colsample_bytree" : 0.8, "verbosity": -1, "metric" : 'rmse'
        }
        params['feature_fraction'] = max(min(feature_fraction, 1), 0)
        params['max_depth'] = int(round(max_depth))
        params['num_leaves'] = int(round(num_leaves))
        params['min_split_gain'] = min_split_gain
        params['min_child_weight'] = min_child_weight
        cv_result = lgb.cv(params, train_data, nfold=n_folds, seed=random_seed, verbose_eval =200,stratified=False)
        return (-1.0 * np.array(cv_result['rmse-mean'])).max()
    
        # range 
    lgbBO = BayesianOptimization(lgb_eval, {'feature_fraction': (0.1, 0.9),
                                            'max_depth': (5, 9),
                                            'num_leaves' : (200,300),
                                            'min_split_gain': (0.001, 0.1),
                                            'min_child_weight': (5, 50)}, random_state=0,verbose=0)
        # optimize
    lgbBO.maximize(init_points=init_round, n_iter=opt_round,acq='ei')
    params = pd.DataFrame(lgbBO.res).sort_values('target', ascending = False).iloc[0,1]
    params['max_depth'] = int(params['max_depth'])
    params['num_leaves'] = int(params['num_leaves'])

    return params#.res['max']['max_params']

"""
{'max_depth': (2, 7), # Máximo 7 para implantação. Não ultrapassar 7!
                                   'gamma': (0.001, 10.0),
                                   'min_child_weight': (0, 20),
                                   'max_delta_step': (0, 10),
                                   'subsample': (0.4, 1.0),
                                   'colsample_bytree':(0.4, 1.0),
                                   'eta': (0.1, 1.0)}
"""
# Random forest 

from bayes_opt import BayesianOptimization
from sklearn.model_selection import cross_val_score
from sklearn.ensemble import RandomForestRegressor

#General function for building a model
def rf_cv(X, y, **kwargs):
    estimator = RandomForestRegressor(**kwargs)
    cval = cross_val_score(estimator, X, y, scoring = 'neg_mean_squared_error', cv = 4, verbose = 0, n_jobs = -1)
    return cval.mean()


def bayesian_optimise_rf(X, y, n_iter = 100):
    def rf_crossval(n_estimators, max_features):
        #Wrapper of RandomForest cross validation.
        #Note the fixing of the inputs so they match the expected type
        #(e.g n_estimators must be an integer)
        return rf_cv(
            X = X,
            y = y,
            n_estimators = int(n_estimators),
            max_features = max(min(max_features, 0.999), 1e-3),
            bootstrap = True
        )
    
    optimizer = BayesianOptimization(verbose=0,
        f = rf_crossval,
        pbounds = {
            "n_estimators" : (10, 400),
            "max_features" : (0.1, 0.999),
        }
    )
    optimizer.maximize(n_iter = n_iter)
    best_params = optimizer.max['params']
    best_params['n_estimators'] = int(best_params['n_estimators'])
    return best_params


"""
--------------------------- damma ------------------------------
Downscaling com aprendizado de maquinas para modelos atmosfericos
Downscaling Atmospheric Model with machine learning

modulo para downscaling de campos de temperatura provindos do wrf utilizando machine learning

"""

from .preprocessing import *
#from .train_preprocessing import *
#from .Train import *
from .posprocessing import *
from .estatisticas import *
from .downscaling import *
#from .DammaRun import *